@@ -1,4 +1,3 @@
-from ..core.object import Object, AccessSpecifier
 
 
 class TranslatorBase(object):
@@ -55,12 +54,6 @@ class TranslatorBase(object):
                 member.initial_value = 'None'
 
             shift += 1
-        value = Object()
-        value.initial_value = '{}::{}'.format(cls.name, cls.members[0].name)
-        value.name = '_value'
-        value.type = cast
-        value.access = AccessSpecifier.private
-        cls.members.append(value)
         return values
 
     @staticmethod
