@@ -1,6 +1,6 @@
 import re
 from ..base import WriterBase
-from ..core.object import Object
+from ..core.object import Object, AccessSpecifier
 from .regex import RegexPatternCpp
 
 
@@ -8,15 +8,13 @@ class Writer(WriterBase):
 
     def __init__(self, out_directory):
         WriterBase.__init__(self, out_directory)
-        self.current_cls = None
         self.objects_cache = {}
-        self.methods_cache = []
+        self.methods_cache = {}
         self.methods_cache_with_templates = []
 
     def write_class(self, cls):
-        self.current_cls = cls
         self.objects_cache = {}
-        self.methods_cache = []
+        self.methods_cache = {}
 
         for member in cls.members:
             declaration, initialisation, static_initialization = self.write_object(member)
@@ -24,7 +22,7 @@ class Writer(WriterBase):
 
         for method in cls.functions:
             hpp, cpp = self.write_function(method)
-            self.methods_cache.append([hpp, cpp])
+            self.methods_cache[method] = ([hpp, cpp])
             if method.template_types:
                 self.methods_cache_with_templates.append(hpp)
 
@@ -40,16 +38,16 @@ class Writer(WriterBase):
         initialization = ''
         static_initialization = ''
 
-        assert (isinstance(obj.type, str))
-        if self.current_cls.type == 'class':
+        assert isinstance(obj.type, str)
+        if self.current_class.type == 'class':
             declaration = self.write_member_declaration(obj)
-        elif self.current_cls.type == 'enum':
+        elif self.current_class.type == 'enum':
             declaration = self.write_member_enum_declaration(obj)
 
-        if self.current_cls.type == 'enum' and obj.name != 'value':
-            static_initialization = self.write_member_static_enum(self.current_cls, obj)
+        if self.current_class.type == 'enum':
+            pass
         elif obj.is_static:
-            static_initialization = self.write_member_static_init(self.current_cls, obj)
+            static_initialization = self.write_member_static_init(self.current_class, obj)
         else:
             initialization += self.write_member_initialization(obj)
         return declaration, initialization, static_initialization
@@ -62,25 +60,44 @@ class Writer(WriterBase):
         class_name = cls.name
         includes, forward_declarations, forward_declarations_out = self.get_includes_for_header(cls)
         functions = ''
-        for method in self.methods_cache:
-            functions += method[0]
+        if cls.type == 'enum':
+            functions += '{name}(const BaseEnum& rhs):BaseEnum(rhs){{}}'.format(name=class_name)
+            functions += '\nconst {name}& operator =(const BaseEnum& rhs) {{ this->value = rhs.operator int(); return *this; }}'.format(name=class_name)
+
+        access = AccessSpecifier.public
+        for method in cls.functions:
+            if method.name == 'constructor':
+                continue
+            hpp = self.methods_cache[method][0]
+            if access != method.access:
+                functions += AccessSpecifier.to_string(method.access) + ':\n'
+                access = method.access
+            functions += hpp
         for method in self.methods_cache_with_templates:
             includes.update(self.get_includes_for_method(cls, method, includes))
         members = ''
+
+        access = AccessSpecifier.public
         for member in cls.members:
+            if access != member.access:
+                members += AccessSpecifier.to_string(member.access) + ':\n'
+                access = member.access
             declaration, _, _ = self.objects_cache[member.name]
             members += declaration + '\n'
 
-        virtual = 'virtual '
-        if not cls.is_virtual and not cls.superclasses and not cls.subclasses:
-            virtual = ''
-        destructor = '{virtual}~{name}();'.format(virtual=virtual, name=cls.name)
+        virtual = 'virtual ' if cls.has_virtual_table() else ''
+        destructor = '{virtual}~{name}();'.format(virtual=virtual, name=cls.name) if cls.type != 'enum' else ''
 
         superclass = '' if not cls.superclasses else ' : public %s' % cls.superclasses[0].name
 
         includes_s = self.build_includes(cls, includes)
         forward_declarations_s = self.build_forward_declarations(forward_declarations)
         forward_declarations_out_s = self.build_forward_declarations(forward_declarations_out)
+
+        constructor_args = ''
+        if cls.constructor is not None:
+            constructor_args = self.create_function_hpp_args_string(cls.constructor)
+
         header = HEADER.format(namespace=namespace,
                                class_name=class_name,
                                destructor=destructor,
@@ -89,15 +106,19 @@ class Writer(WriterBase):
                                forward_declarations_out=forward_declarations_out_s,
                                functions=functions,
                                members=members,
-                               superclass=superclass)
+                               superclass=superclass,
+                               constructor_args=constructor_args)
         return header, includes, forward_declarations, forward_declarations_out
 
     def write_cpp(self, cls, includes, forw_declarations, forw_declarations_out):
         namespace = 'mg'
         class_name = cls.name
         functions = ''
-        for method in self.methods_cache:
-            functions += method[1]
+        for method in cls.functions:
+            if method.name == 'constructor':
+                continue
+            cpp = self.methods_cache[method][1]
+            functions += cpp
         static_initializations = ''
         initializations = ''
         for member in cls.members:
@@ -111,10 +132,9 @@ class Writer(WriterBase):
         destructor = '''{name}::~{name}()
         {{
         }}
-        '''.format(name=cls.name)
+        '''.format(name=cls.name) if cls.type != 'enum' else ''
 
-        registration = 'REGISTRATION_OBJECT({0});\n'.format(cls.name)
-        has_get_type = False
+        registration = 'REGISTRATION_OBJECT({});\n'.format(cls.name) if self.model.auto_registration else ''
         is_abstract = cls.is_abstract
         if not is_abstract:
             for method in cls.functions:
@@ -123,14 +143,17 @@ class Writer(WriterBase):
                     break
         if is_abstract:
             registration = ''
-        for method in cls.functions:
-            if method.name == 'get_type':
-                has_get_type = True
-        if not has_get_type:
+        if registration and not cls.has_method_with_name('get_type'):
             registration = ''
 
         includes = self.get_includes_for_source(cls, functions, includes, forw_declarations, forw_declarations_out)
 
+        constructor_args = ''
+        constructor_body = ''
+        if cls.constructor is not None:
+            constructor_args = self.create_function_cpp_args_string(cls.constructor)
+            constructor_body = cls.constructor.body
+
         return SOURCE.format(namespace=namespace,
                              path_to_root=Writer.get_path_to_root(cls),
                              class_name=class_name,
@@ -139,23 +162,26 @@ class Writer(WriterBase):
                              functions=functions,
                              static_initializations=static_initializations,
                              initializations=initializations,
-                             registration=registration)
-
-    def write_function_hpp(self, method):
-        string = '{virtual}{static}{type} {name}({args}){const}{override}{abstract}'
+                             registration=registration,
+                             constructor_args=constructor_args,
+                             constructor_body=constructor_body)
 
+    def create_function_hpp_args_string(self, method):
         args = list()
         for arg in method.args:
-            assert(isinstance(arg[1], Object))
+            assert isinstance(arg[1], Object)
             args.append(self.write_named_object(arg[1], arg[0], True, False))
             if arg[1].initial_value is not None:
                 args[-1] += '=' + self.convert_initial_value(arg[1])
         args = ', '.join(args)
+        return args
 
-        assert (isinstance(method.return_type, Object))
+    def write_function_hpp(self, method):
+        string = '{virtual}{static}{friend}{type} {name}({args}){const}{override}{abstract}'
+        assert isinstance(method.return_type, Object)
         return_type = self.write_named_object(method.return_type, '', False, True)
-
-        virtual = 'virtual ' if method.is_virtual or method.is_abstract or self.current_cls.is_virtual else ''
+        args = self.create_function_hpp_args_string(method)
+        virtual = 'virtual ' if method.is_virtual or method.is_abstract or self.current_class.is_virtual else ''
 
         body = ''
         if method.template_types:
@@ -165,23 +191,32 @@ class Writer(WriterBase):
             {body}
             }}
             '''
-            templates = ['class %s'%x for x in method.template_types]
+            templates = ['class %s' % x for x in method.template_types]
             templates = ', '.join(templates)
-            templates = 'template<%s> '%templates
+            templates = 'template<%s> ' % templates
             string = templates + string
         else:
             string += ';\n'
 
         return string.format(virtual=virtual,
                              static='static ' if method.is_static else '',
+                             friend='friend ' if method.is_friend else '',
                              const=' const' if method.is_const else '',
-                             override='',
                              abstract=' = 0' if method.is_abstract else '',
+                             override='',
                              type=return_type,
                              name=method.name,
                              args=args,
                              body=body)
 
+    def create_function_cpp_args_string(self, method):
+        args = list()
+        for arg in method.args:
+            assert isinstance(arg[1], Object)
+            args.append(self.write_named_object(arg[1], arg[0], True, False))
+        args = ', '.join(args)
+        return args
+
     def write_function_cpp(self, method):
         if method.is_external or method.is_abstract:
             return ''
@@ -190,19 +225,15 @@ class Writer(WriterBase):
         if method.specific_implementations:
             return method.specific_implementations
 
-        text = '''{type} {class_name}::{name}({args}){const}
+        scope = (self.current_class.name + '::') if not method.is_friend else ''
+        text = '''{type} {scope}{name}({args}){const}
         {{
         {body}
         }}
         
         '''
         return_type = self.write_named_object(method.return_type, '', False, True)
-
-        args = list()
-        for arg in method.args:
-            assert(isinstance(arg[1], Object))
-            args.append(self.write_named_object(arg[1], arg[0], True, False))
-        args = ', '.join(args)
+        args = self.create_function_cpp_args_string(method)
 
         body = method.body
 
@@ -210,7 +241,7 @@ class Writer(WriterBase):
                            type=return_type,
                            name=method.name,
                            args=args,
-                           class_name=self.current_cls.name,
+                           scope=scope,
                            body=body)
 
     def write_member_declaration(self, obj):
@@ -219,9 +250,8 @@ class Writer(WriterBase):
     def write_member_enum_declaration(self, obj):
         if obj.name == 'value':
             return self.write_member_declaration(obj)
-        return 'static constexpr {type} {name} = {value};'.format(type=obj.type,
-                                                                  name=obj.name,
-                                                                  value=obj.initial_value)
+        return 'static constexpr BaseEnum {name} = {value};'.format(name=obj.name,
+                                                                    value=obj.initial_value)
 
     def write_member_static_init(self, cls, obj, use_intrusive=True):
         string_pointer = '{const}intrusive_ptr<{type}> {owner}::{name}({initial_value});'
@@ -240,15 +270,14 @@ class Writer(WriterBase):
                              initial_value=self.convert_initial_value(obj))
 
     def write_member_static_enum(self, cls, obj):
-        assert (self is not None)
-        string = 'const int {owner}::{name};'
-        return string.format(owner=cls.name, name=obj.name)
+        assert self is not None
+        return 'const int {}::{};'.format(cls.name, obj.name)
 
     def convert_initial_value(self, object_):
         if object_.is_pointer and (object_.initial_value == '0' or object_.initial_value is None):
             return 'nullptr'
         type_class = self.model.get_class(object_.type) if self.model.has_class(object_.type) else None
-        if type_class is not None and type_class.type == 'enum':
+        if type_class is not None and type_class.type == 'enum' and object_.initial_value is None:
             assert type_class.members
             return '{}::{}'.format(type_class.name, type_class.members[0].name)
 
@@ -257,23 +286,29 @@ class Writer(WriterBase):
         return object_.initial_value
 
     def write_member_initialization(self, obj):
-        string = '{name}({initial_value})'
         initial_value = self.convert_initial_value(obj)
-        return string.format(name=obj.name, initial_value=initial_value)
+        return '{}({})'.format(obj.name, initial_value)
 
     @staticmethod
     def write_named_object(obj, name, try_to_use_const_ref, use_intrusive):
 
         def can_use_const_ref(object_):
-            assert (isinstance(object_, Object))
+            assert isinstance(object_, Object)
             return object_.type in ['string', 'list', 'map']
 
         string_non_pointer = '{static}{const}{type}{templates}{pointer}{ref}{name}'
         string_pointer = '{static}{const}intrusive_ptr<{type}{templates}>{ref}{name}'
         templates = []
         for arg in obj.template_args:
-            assert(isinstance(arg, Object))
+            assert isinstance(arg, Object)
             templates.append(Writer.write_named_object(arg, '', False, True))
+            if arg.callable_args is not None:
+                callable_args = []
+                for callable_arg in arg.callable_args:
+                    callable_args.append(Writer.write_named_object(callable_arg, '', True, False))
+                callable_args = '({})'.format(', '.join(callable_args))
+                templates[-1] += callable_args
+
         templates = ('<' + ', '.join(templates) + '>') if templates else ''
         is_ref = obj.is_ref
         is_const = obj.is_const
@@ -285,11 +320,11 @@ class Writer(WriterBase):
         if '(' in modified_type and ')' in modified_type:
             left = modified_type.index('(')
             right = modified_type.index(')')
-            args = modified_type[left+1:right]
+            args = modified_type[left + 1:right]
             args = args.split(',')
             args = [Writer.convert_type(x.strip()) for x in args]
             args = ', '.join(args)
-            modified_type = modified_type[0:left+1] + args + modified_type[right:]
+            modified_type = modified_type[0:left + 1] + args + modified_type[right:]
 
         if use_intrusive and obj.is_pointer and not obj.is_const and not obj.is_link and not obj.denied_intrusive:
             string = string_pointer
@@ -305,7 +340,7 @@ class Writer(WriterBase):
 
     @staticmethod
     def convert_type(type_of_object):
-        assert (isinstance(type_of_object, str))
+        assert isinstance(type_of_object, str)
         types = {
             'list': 'std::vector',
             'map': 'std::map',
@@ -328,17 +363,18 @@ class Writer(WriterBase):
         for line in lines:
             line = line.strip()
 
+            backward = False
             if line and line[0] == '}':
                 tabs -= 1
-            backward = False
-            if 'public:' in line or 'protected:' in line or 'private:' in line:
+            elif 'public:' in line or 'protected:' in line or 'private:' in line:
                 backward = True
                 tabs -= 1
+            line_strip = line
             if line:
                 line = get_tabs(tabs) + line
             if backward:
                 tabs += 1
-            if line.strip() and line.strip()[0] == '{':
+            if line_strip and line_strip[0] == '{':
                 tabs += 1
             text.append(line)
         text = '\n'.join(text)
@@ -361,6 +397,9 @@ class Writer(WriterBase):
             set_.add(self.convert_type(obj.type))
             for arg in obj.template_args:
                 add(forward_declarations, arg)
+            if obj.callable_args is not None:
+                for arg in obj.callable_args:
+                    add(forward_declarations, arg)
 
         # members
         for member in cls.members:
@@ -407,25 +446,20 @@ class Writer(WriterBase):
         return self.build_includes(cls, includes)
 
     def get_includes_for_method(self, cls, functions_text, hpp_includes):
-        includes = set()
+        includes = list()
 
-        def add(set_, obj):
-            set_.add(self.convert_type(obj.type))
-            for arg in obj.template_args:
-                add(includes, arg)
-
-        if 'DataStorage::shared' in functions_text:
-            includes.add('DataStorage')
         for type_ in self.model.classes:
             name = type_.name
             if name not in hpp_includes and name in functions_text:
+                pattern = None
                 if name not in RegexPatternCpp.regs_class_names:
-                    RegexPatternCpp.regs_class_names[name] = re.compile(r'\b{}\b'.format(name))
-                pattern = RegexPatternCpp.regs_class_names[name]
-                need = cls.name == name
+                    pattern = re.compile(r'\b{}\b'.format(name))
+                    RegexPatternCpp.regs_class_names[name] = pattern
+                pattern = pattern or RegexPatternCpp.regs_class_names[name]
+                need = cls.name is name
                 need = need or pattern.search(functions_text) is not None
                 if need:
-                    includes.add(name)
+                    includes.append(name)
         return includes
 
     def build_includes(self, cls, includes):
@@ -442,7 +476,7 @@ class Writer(WriterBase):
         }
         result = []
         for typename in includes:
-            assert(isinstance(typename, str))
+            assert isinstance(typename, str)
             if typename in types:
                 result.append('#include %s' % types[typename])
                 continue
@@ -456,18 +490,18 @@ class Writer(WriterBase):
         return '\n'.join(result)
 
     @staticmethod
-    def get_include_path_to_class(cls, to_cls):
+    def get_include_path_to_class(cls_, to_cls):
         include = '"'
-        if cls.group and to_cls.group != cls.group:
+        if cls_.group and to_cls.group != cls_.group:
             include += '../'
-        if to_cls.group and to_cls.group != cls.group:
+        if to_cls.group and to_cls.group != cls_.group:
             include += to_cls.group + '/'
         include += to_cls.name + '.h"'
         return include
 
     @staticmethod
-    def get_path_to_root(cls):
-        if cls.group:
+    def get_path_to_root(cls_):
+        if cls_.group:
             return '../'
         return ''
 
@@ -509,12 +543,16 @@ HEADER = '''#ifndef __{namespace}_{class_name}_h__
 {forward_declarations_out}
 namespace {namespace}
 {{
+    class SerializerXml;
+    class DeserializerXml;
+    class SerializerJson;
+    class DeserializerJson;
 {forward_declarations}
 
     class {class_name}{superclass}
     {{
     public:
-        {class_name}();
+        {class_name}({constructor_args});
         {destructor}
 {functions}
 {members}
@@ -529,13 +567,16 @@ SOURCE = '''
 #include "{path_to_root}{namespace}_Factory.h"
 {includes}
 #include "{path_to_root}{namespace}_extensions.h"
+#include "{path_to_root}SerializerJson.h"
+#include "{path_to_root}SerializerXml.h"
 
 namespace {namespace}
 {{
     {static_initializations}
     {registration}
-    {class_name}::{class_name}()
+    {class_name}::{class_name}({constructor_args})
     {initializations}{{
+    {constructor_body}
     }}
     
     {destructor}
