@@ -37,6 +37,7 @@ class RegexPatternPython(object):
         (re.compile(r'list_insert\s*\(\s*(.+?),\s*(.+)\s*,\s*(.+)\s*\)'), r'\1.insert(\2, \3)', ['list_insert']),
         (re.compile(r'list_remove\s*\(\s*(.+),\s*(.+)\s*\)'), r'\1.remove(\2)', ['list_remove']),
         (re.compile(r'list_erase\s*\(\s*(.+),\s*(.+)\s*\)'), r'\1.remove(\1[\2])', ['list_erase']),
+        (re.compile(r'list_truncate\s*\(\s*(.+),\s*(.+)\s*\)'), r'\1 = \1[:\2]', ['list_truncate']),
         (re.compile(r'list_clear\s*\(\s*(.+)\s*\)'), r'\1 = list()', ['list_clear']),
         (re.compile(r'list_size\s*\('), r'len(', ['list_size']),
         (re.compile(r'list_resize\s*\(\s*(.+),\s*(.+)\s*\)'), r'\1 = [None for _ in range(int(\2))]', ['list_resize']),
@@ -52,8 +53,13 @@ class RegexPatternPython(object):
         (re.compile(r'std::string\s+(\w+)'), r'\1', ['std::string']),
         (re.compile(r'\blist<.+>\s+(\w+)'), r'\1 = list()', ['list<']),
         (re.compile(r'\bmap<([<:>\w\s\*&]+),\s*([<:>\w\s\*&]+)>\s*(\w+)'), r'\3 = dict()', ['map<']),
+
+        # nullish coalescing operator
+        # auto c = test->a ?? test->b;
+        # https://trello.com/c/WPR70BUY/37-add-operator
         (re.compile(r'[\w\*]+ ([\w\->\.]+)\s*=\s*([\w\->\.]+)\s*\?\?\s*([\w\->\.]+)'),
          r'\1 = (\2) if (\2 is not None) else (\3)', ['??']),
+
         (re.compile(r'\bauto\&* (\w+)'), r'\1', ['auto']),
         (re.compile(r'\bstring (\w+)'), r'\1', ['string']),
         (re.compile(r'\bint (\w+)'), r'\1', ['int']),
@@ -71,8 +77,9 @@ class RegexPatternPython(object):
         (re.compile(r'&(\w+)'), r'\1'),
         (re.compile(r'!(\w+)'), r'not \1'),
         (re.compile(r'!\('), r'not ('),
-        (re.compile(r'make_intrusive<(\w+)>\(\)'), r'\1()', ['make_intrusive']),
-        (re.compile(r'new\s*(\w+)\s*\((.*?)\)'), r'\1(\2)', ['new']),
+        (re.compile(r'make_intrusive<(\w+)>\(\)'), r'make_intrusive(\1)', ['make_intrusive']),
+        (re.compile(r'new\s*(\w+)\s*\(\s*?\)'), r'make_intrusive(\1)', ['new']),
+        (re.compile(r'new\s*(\w+)\s*\((.*?)\)'), r'make_intrusive(\1, \2)', ['new']),
         (re.compile(r'assert\(.+\);'), r'', ['assert ']),
         (re.compile(r'(\b[-0-9]+)\.f\b'), r'\1.0'),
         (re.compile(r'(\b[-0-9]+)\.([-0-9]*)f\b'), r'\1.\2'),
@@ -97,6 +104,7 @@ class RegexPatternPython(object):
         (re.compile(r'try\n\s*{([\s\S.]+?)}\n\s*catch\(((\w+)\s*(\w*))\)\n\s+{([\s\S.]+?)}'),
          r'try:\n{\n\1\n}\nexcept BaseException as \4:\n{\n\5\n}\n', ['try']),
         (re.compile(r'throw Exception\((.*?)\)'), r'raise BaseException()', ['throw']),
+        (re.compile(r'throw make_intrusive\(Exception\)'), r'raise BaseException()', ['throw']),
     )
 
     PEP8 = (
