@@ -26,7 +26,13 @@ class Writer(WriterBase):
             text = self.write_function(method)
             functions += text
 
-        imports = ''
+        imports = 'from .common import *\n'
+        if not cls.type == 'enum' and cls.name != 'BaseEnum':
+            imports += 'from .SerializerXml import SerializerXml\n'
+            imports += 'from .DeserializerXml import DeserializerXml\n'
+            imports += 'from .SerializerJson import SerializerJson\n'
+            imports += 'from .DeserializerJson import DeserializerJson\n'
+            imports += 'from .IntrusivePtr import *\n'
         init_superclass = ''
         name = cls.name
         if cls.superclasses:
