@@ -15,7 +15,7 @@ from mlc_tools.base.linker import Linker
 from mlc_tools.base.validator import Validator
 
 
-def create_test_model():
+def create_test_model(with_ctr=False):
     cls = Class()
     cls.name = 'Test'
 
@@ -37,6 +37,16 @@ def create_test_model():
     ]
     cls.functions.append(f)
 
+    if with_ctr:
+        ctr = Function()
+        ctr.name = 'const'
+        ctr.name += 'ructor'
+        ctr.return_type = Object()
+        ctr.args = []
+        ctr.operations = ['this->int_value = 1;']
+        cls.functions.append(ctr)
+        cls.generate_constructor()
+
     model = Model()
     # model.parser = Parser(model)
     model.add_class(cls)
@@ -61,21 +71,22 @@ def create_lang(lang, model):
 
 
 def save(lang, model):
-    create_lang(lang, model).get_writer().save(model)
-    model.save_files()
+    lang = create_lang(lang, model)
+    lang.get_writer().save(model)
+    lang.save_plugin.save_files()
 
 
 def save_object(lang, model):
     cls = model.get_class('Test')
     writer = create_lang(lang, model).get_writer()
-    writer.current_cls = cls
+    writer.current_class = cls
     return writer.write_object(cls.members[0])
 
 
 def save_function(lang, model):
     cls = model.get_class('Test')
     writer = create_lang(lang, model).get_writer()
-    writer.current_cls = cls
+    writer.current_class = cls
     return writer.write_function(cls.functions[0])
 
 
@@ -90,7 +101,19 @@ class TestWriteClass(unittest.TestCase):
 
         hpp = model.out_dict['Test.h']
         cpp = model.out_dict['Test.cpp']
-        self.assertEqual(cpp, get_cpp())
+        self.assertEqual(cpp, get_cpp(False))
+        self.assertEqual(hpp, get_hpp())
+
+    def test_cpp_with_ctr(self):
+        model = create_test_model(True)
+        save('cpp', model)
+
+        self.assertTrue('Test.h' in model.out_dict)
+        self.assertTrue('Test.cpp' in model.out_dict)
+
+        hpp = model.out_dict['Test.h']
+        cpp = model.out_dict['Test.cpp']
+        self.assertEqual(cpp, get_cpp(True))
         self.assertEqual(hpp, get_hpp())
 
 
@@ -137,7 +160,7 @@ return result;
     def test_php(self):
         model = create_test_model()
         result = save_function('php', model)
-        self.assertEqual(result, 'function foo($a0, $a1)\n{\n    $result = $this->int_value;\n$result += $a0;\nreturn $result;\n\n}\n')
+        self.assertEqual(result, 'public function foo(int $a0, string $a1)\n{\n    $result = $this->int_value;\n$result += $a0;\nreturn $result;\n\n}\n')
 
 
 class TestCppWriterBuildIncludesWithGroups(unittest.TestCase):
@@ -205,12 +228,62 @@ class TestCppWriterWriteObject(unittest.TestCase):
                          'intrusive_ptr<DataBase> Test::db(nullptr);')
 
 
+class TestWriterPrepareFile(unittest.TestCase):
 
-def get_cpp():
+    def test_xml(self):
+        from mlc_tools.base.model import SerializeFormat
+        writer = self.create_writer()
+        writer.model.serialize_formats = SerializeFormat.xml
+        text = writer.prepare_file(TestWriterPrepareFile.get_text())
+        self.assertIn('xml_functional', text)
+        self.assertNotIn('json_functional', text)
+        self.assertNotIn('both_functional', text)
+        self.assertNotIn('format=', text)
+
+    def test_json(self):
+        from mlc_tools.base.model import SerializeFormat
+        writer = self.create_writer()
+        writer.model.serialize_formats = SerializeFormat.json
+        text = writer.prepare_file(TestWriterPrepareFile.get_text())
+        self.assertNotIn('xml_functional', text)
+        self.assertIn('json_functional', text)
+        self.assertNotIn('both_functional', text)
+        self.assertNotIn('format=', text)
+
+    def test_both(self):
+        from mlc_tools.base.model import SerializeFormat
+        writer = self.create_writer()
+        writer.model.serialize_formats = SerializeFormat.xml | SerializeFormat.json
+        text = writer.prepare_file(TestWriterPrepareFile.get_text())
+        self.assertNotIn('xml_functional', text)
+        self.assertNotIn('json_functional', text)
+        self.assertIn('both_functional', text)
+        self.assertNotIn('format=', text)
+
+    @staticmethod
+    def create_writer():
+        from mlc_tools.base.writer_base import WriterBase
+        writer = WriterBase('')
+        writer.model = Model()
+        return writer
+
+    @staticmethod
+    def get_text():
+        text = '''
+        {{format=xml}}xml_functional{{end_format=xml}}
+        {{format=json}}json_functional{{end_format=json}}
+        {{format=both}}both_functional{{end_format=both}}
+        '''
+        return text
+
+
+def get_cpp(with_ctr):
     return '''#include "intrusive_ptr.h"
 #include "mg_Factory.h"
 #include "Test.h"
 #include "mg_extensions.h"
+#include "SerializerJson.h"
+#include "SerializerXml.h"
 
 namespace mg
 {
@@ -221,6 +294,7 @@ namespace mg
     Test::Test()
     : int_value(42)
     {
+{ctr}
     }
 
     Test::~Test()
@@ -238,15 +312,13 @@ namespace mg
     bool Test::operator ==(const Test& rhs) const
     {
         bool result = true;
-        result = result && int_value == rhs.int_value;
+        result = result && this->int_value == rhs.int_value;
         return result;
     }
 
     bool Test::operator !=(const Test& rhs) const
     {
-        bool result = false;
-        result = result || int_value != rhs.int_value;
-        return result;
+        return !(*this == rhs);
     }
 
     std::string Test::get_type() const
@@ -255,7 +327,7 @@ namespace mg
     }
 
 } //namespace mg
-'''
+'''.replace('{ctr}', '        this->int_value = 1;' if with_ctr else '')
 
 
 def get_hpp():
@@ -268,6 +340,10 @@ def get_hpp():
 
 namespace mg
 {
+    class SerializerXml;
+    class DeserializerXml;
+    class SerializerJson;
+    class DeserializerJson;
 
     class Test
     {
@@ -289,5 +365,9 @@ namespace mg
 '''
 
 
+def get_hpp_with_ctr():
+    return ''''''
+
+
 if __name__ == '__main__':
     unittest.main()
